#!/usr/bin/env python

"""
State management classes for iterators.

Copyright (C) 2011 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from os.path import commonprefix

class CommonPrefixState:

    "A class whose instances maintain common prefix state."

    def __init__(self, initial=""):
        self.initial = initial
        self.reset()

    def reset(self):
        self.value = self.initial

class CommonPrefixDecoder(CommonPrefixState):

    "A class whose instances decode common prefix information."

    def update(self, common_plus_suffix):
        common, suffix = common_plus_suffix
        self.value = self.value[:common] + suffix
        return self.value

class CommonPrefixEncoder(CommonPrefixState):

    "A class whose instances encode common prefix information."

    def update(self, value):
        common = len(commonprefix((self.value, value)))
        suffix = value[common:]
        self.value = value
        return common, suffix

# vim: tabstop=4 expandtab shiftwidth=4
